<?php

namespace Drupal\entity_sync\Entity\Storage;

use Drupal\Core\Entity\EntityTypeManagerInterface;

/**
 * The default implementation of the configuration entity loader.
 */
class ConfigEntityLoader implements ConfigEntityLoaderInterface {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a new ConfigEntityLoader object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public function load(string $sync_id) {
    $sync = $this->entityTypeManager->getStorage('entity_sync')->load($sync_id);
    if ($sync) {
      return $sync;
    }

    return $this->entityTypeManager
      ->getStorage('entity_sync_operation_type')
      ->loadWithPluginInstantiated($sync_id);
  }

  /**
   * {@inheritdoc}
   */
  public function loadMultipleWithPluginInstantiated(array $sync_ids = NULL) {
    $syncs = $this->entityTypeManager
      ->getStorage('entity_sync')
      ->loadMultiple($sync_ids);

    $operation_types = $this->entityTypeManager
      ->getStorage('entity_sync_operation_type')
      ->loadMultipleWithPluginInstantiated($sync_ids);

    // Results are indexed by entity ID. Using intentionally the `+` operator so
    // that operation types that have the same IDs as synchronization entities
    // are skipped.
    return $syncs + $operation_types;
  }

}
