<?php

namespace Drupal\entity_sync\EventSubscriber;

use Drupal\entity_sync\Export\Event\Events;
use Drupal\entity_sync\Export\Event\LocalEntityMappingEvent;
use Drupal\entity_sync\Export\EntityManagerInterface;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Builds the default remote ID mapping for export operations.
 */
class DefaultExportLocalEntityMapping implements EventSubscriberInterface {

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events = [
      Events::LOCAL_ENTITY_MAPPING => ['buildEntityMapping', 0],
    ];
    return $events;
  }

  /**
   * Builds the default local entity mapping.
   *
   * The default local entity mapping is defined in the synchronization
   * or operation type configuration object.
   *
   * @param \Drupal\entity_sync\Export\Event\LocalEntityMappingEvent $event
   *   The local entity mapping event.
   */
  public function buildEntityMapping(LocalEntityMappingEvent $event) {
    $entity_mapping = [];

    $sync = $event->getSync();
    $local_entity = $event->getLocalEntity();

    // We can only provide the default mapping if we have a remote ID
    // field. Otherwise, an alternative subscriber must be provided by a
    // custom/contrib module. If a remote ID field is defined in the
    // configuration but it does not exist on the entity, we let an error to be
    // raised as that must be an error in development/configuration.
    $id_field_name = $sync->getLocalEntitySettings()['remote_id_field'] ?? NULL;
    if ($id_field_name === NULL) {
      return;
    }

    // If we don't have a value it's a create, if we do it's an update.
    $id_field = $local_entity->get($id_field_name);
    $remote_resource_settings = $sync->getRemoteResourceSettings();
    if ($id_field->isEmpty()) {
      $entity_mapping = [
        'action' => EntityManagerInterface::ACTION_CREATE,
        'client' => $remote_resource_settings['client'] ?? NULL,
      ];
    }
    else {
      $entity_mapping = [
        'action' => EntityManagerInterface::ACTION_UPDATE,
        'client' => $remote_resource_settings['client'] ?? NULL,
        'id' => $id_field->value,
      ];
    }

    $event->setEntityMapping($entity_mapping);
  }

}
