<?php

namespace Drupal\entity_sync\Plugin\EntitySync\FieldTransformer;

use Drupal\entity_sync\FieldTransformer\PluginBase;
use Drupal\Core\Entity\ContentEntityInterface;

/**
 * Transformer that converts an entity to the value of one of its fields.
 *
 * Supported configuration properties:
 * - field_name: (string, required) The name of the field from which to extract
 *   the value.
 *
 * Currently, only exports are supported. Support for imports should be added
 * here as well.
 *
 * @EntitySyncFieldTransformer(
 *   id = "entity_field"
 * )
 */
class EntityField extends PluginBase {

  /**
   * {@inheritdoc}
   */
  protected function validateConfiguration() {
    $this->validateConfigurationRequiredProperties([
      'field_name',
    ]);
  }

  /**
   * {@inheritdoc}
   */
  protected function transformExportedValue(
    $entity,
    ContentEntityInterface $local_entity,
    $remote_entity_id,
    array $field_info,
    array $context
  ) {
    if ($entity === NULL) {
      return NULL;
    }

    // We can take advantage of the field-extracting functionality provided by
    // the base field transformer class. This will also allow us to make use of
    // other functionality such as property and delta extracting.
    return $this->extractFieldValue(
      $entity,
      ['machine_name' => $this->configuration['field_name']]
    );
  }

}
